package com.dayou.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.dayou.dto.AddressDTO;
import com.dayou.dto.PersonalFacePriceQueryDTO;
import com.dayou.mapper.PersonalFacePriceMapper;
import com.dayou.service.IExternalDyDataBaseService;
import com.dayou.service.IPersonalFacePriceService;
import com.dayou.utils.AddressUtil;
import com.dayou.utils.DateUtils;
import com.dayou.vo.DayouPersonalPriceVO;
import com.dayou.vo.ExternalPersonalVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class PersonalFacePriceServiceImpl implements IPersonalFacePriceService {

    @Autowired
    private PersonalFacePriceMapper personalFacePriceMapper;

    @Autowired
    private IExternalDyDataBaseService externalDyDataBaseService;

    @Override
    public List<DayouPersonalPriceVO> dayouPersonalPrice(PersonalFacePriceQueryDTO facePriceQuery) {
        //获取配置的价值点范围
        Date startDate = DateUtils.addYears(new Date(),-1);
        //查询策略1：全地址查询
        String location = facePriceQuery.getLocation();
        AddressDTO targetAddress = AddressUtil.parseAddress(location);
        String roadNumberAddress = AddressUtil.getRoadNumberAddress(location);
        List<DayouPersonalPriceVO> byRoadNumberResult = personalFacePriceMapper.queryByFuzzyAddress(roadNumberAddress,startDate);
        byRoadNumberResult.stream().forEach(x->x.setMatching("全地址匹配"));

        //查询策略2：路名路号查询
        List<DayouPersonalPriceVO> byRoadNameNumberResult = new ArrayList<>();
        String roadAndRoadNumber = AddressUtil.getRoadAndRoadNumber(location);
        if (StrUtil.isNotBlank(roadAndRoadNumber)){
            byRoadNameNumberResult = personalFacePriceMapper.queryByFuzzyAddress(roadAndRoadNumber,startDate);
            filterByCityAndDistrict(byRoadNameNumberResult,targetAddress);
        }
        byRoadNameNumberResult.stream().forEach(x->x.setMatching("路名路号匹配"));


        //查询策略3：尝试用小区名字查询
        List<DayouPersonalPriceVO> byCommunityNameResult = new ArrayList<>();
        String communityNameFromAddress = AddressUtil.getCommunityNameFromAddress(location);
        if (StrUtil.isNotBlank(communityNameFromAddress)){
            byCommunityNameResult = personalFacePriceMapper.queryByFuzzyAddress(communityNameFromAddress,startDate);
            filterByCityAndDistrict(byCommunityNameResult,targetAddress);
        }
        byCommunityNameResult.stream().forEach(x->x.setMatching("小区名字匹配"));


        //合并查询结果集
        byRoadNumberResult.addAll(byRoadNameNumberResult);
        byRoadNumberResult.addAll(byCommunityNameResult);

        //去重
        byRoadNumberResult = byRoadNumberResult.stream().distinct().collect(Collectors.toList());

        //排序
        Collections.sort(byRoadNumberResult);
        return byRoadNumberResult;
    }

    @Override
    public List<DayouPersonalPriceVO> externalPersonalPrice(PersonalFacePriceQueryDTO facePriceQuery) {
        //获取配置的价值点范围
        Date startDate = DateUtils.addYears(new Date(),-1);
        String location = facePriceQuery.getLocation();

        AddressDTO targetAddress = AddressUtil.parseAddress(location);

        //获取全路名 查询小区名称
        String roadNumberAddress = AddressUtil.getRoadNumberAddress(location);
        List<String> communityName1 = personalFacePriceMapper.queryCommunityNameByFuzzyAddress(roadNumberAddress);

        //获取路名路号
        String roadAndRoadNumber = AddressUtil.getRoadAndRoadNumber(location);
        List<String> communityName2 = personalFacePriceMapper.queryCommunityNameByFuzzyAddress(roadAndRoadNumber);

        //获取小区名字 从地址中获取
        String communityNameFromAddress = AddressUtil.getCommunityNameFromAddress(location);

        communityName1.addAll(communityName2);

        if (StrUtil.isNotBlank(communityNameFromAddress)){
            communityName1.add(communityNameFromAddress);
        }

        List<String> communityNames = communityName1.stream().distinct().collect(Collectors.toList());
        List<ExternalPersonalVO> externalPersonalVOS = externalDyDataBaseService.externalPriceByCommunityName(communityNames);
        return null;
    }


    /**
     * 移除路名相同，但行政区域不同的元素
     * @param priceList
     */
    private void filterByCityAndDistrict(List<DayouPersonalPriceVO> priceList, AddressDTO targetAddress){
        Iterator<DayouPersonalPriceVO> iterator = priceList.iterator();
        while (iterator.hasNext()) {
            DayouPersonalPriceVO personalPriceVO = iterator.next();
            AddressDTO item = AddressUtil.parseAddress(personalPriceVO.getLocation());
            if (StrUtil.isNotBlank(targetAddress.getCity())){
                String city = item.getCity();
                if (StrUtil.isNotBlank(city) && !targetAddress.getCity().contains(city)){
                    iterator.remove();
                }
            }
            if (StrUtil.isNotBlank(targetAddress.getDistrict())){
                String district = item.getDistrict();
                if (StrUtil.isNotBlank(district) && !targetAddress.getDistrict().contains(district)){
                    iterator.remove();
                }
            }

        }
    }

    /**
     * 根据小区名字在个贷评估历史中查询小区名字
     * @param address
     * @return
     */
    private String getCommunityNameByAddress(String address){
        String roadNumberAddress = AddressUtil.getRoadNumberAddress(address);
       // personalFacePriceMapper.queryCommuintyName(roadNumberAddress);
        return null;
    }
}
